from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import I386SysVModel


class I386SysVFclose(Fclose, I386SysVModel):
    pass


class I386SysVFeof(Feof, I386SysVModel):
    pass


class I386SysVFerror(Ferror, I386SysVModel):
    pass


class I386SysVFgetc(Fgetc, I386SysVModel):
    pass


class I386SysVFgets(Fgets, I386SysVModel):
    pass


class I386SysVFopen(Fopen, I386SysVModel):
    pass


class I386SysVFprintf(Fprintf, I386SysVModel):
    pass


class I386SysVFputc(Fputc, I386SysVModel):
    pass


class I386SysVFputs(Fputs, I386SysVModel):
    pass


class I386SysVFread(Fread, I386SysVModel):
    pass


class I386SysVFscanf(Fscanf, I386SysVModel):
    pass


class I386SysVFseek(Fseek, I386SysVModel):
    pass


class I386SysVFtell(Ftell, I386SysVModel):
    pass


class I386SysVFwrite(Fwrite, I386SysVModel):
    pass


class I386SysVGetc(Getc, I386SysVModel):
    pass


class I386SysVGetchar(Getchar, I386SysVModel):
    pass


class I386SysVGets(Gets, I386SysVModel):
    pass


class I386SysVPrintf(Printf, I386SysVModel):
    pass


class I386SysVPutc(Putc, I386SysVModel):
    pass


class I386SysVPutchar(Putchar, I386SysVModel):
    pass


class I386SysVPuts(Puts, I386SysVModel):
    pass


class I386SysVRemove(Remove, I386SysVModel):
    pass


class I386SysVRename(Rename, I386SysVModel):
    pass


class I386SysVRewind(Rewind, I386SysVModel):
    pass


class I386SysVScanf(Scanf, I386SysVModel):
    pass


class I386SysVSnprintf(Snprintf, I386SysVModel):
    pass


class I386SysVSprintf(Sprintf, I386SysVModel):
    pass


class I386SysVSscanf(Sscanf, I386SysVModel):
    pass


class I386SysVClearerr(Clearerr, I386SysVModel):
    pass


class I386SysVFflush(Fflush, I386SysVModel):
    pass


class I386SysVFreopen(Freopen, I386SysVModel):
    pass


class I386SysVFgetpos(Fgetpos, I386SysVModel):
    pass


class I386SysVFsetpos(Fsetpos, I386SysVModel):
    pass


class I386SysVTmpfile(Tmpfile, I386SysVModel):
    pass


class I386SysVTmpnam(Tmpnam, I386SysVModel):
    pass


class I386SysVUngetc(Ungetc, I386SysVModel):
    pass


class I386SysVVfprintf(Vfprintf, I386SysVModel):
    pass


class I386SysVVfscanf(Vfscanf, I386SysVModel):
    pass


class I386SysVVprintf(Vprintf, I386SysVModel):
    pass


class I386SysVVscanf(Vscanf, I386SysVModel):
    pass


class I386SysVVsnprintf(Vsnprintf, I386SysVModel):
    pass


class I386SysVVsprintf(Vsprintf, I386SysVModel):
    pass


class I386SysVVsscanf(Vsscanf, I386SysVModel):
    pass


__all__ = [
    "I386SysVFclose",
    "I386SysVFeof",
    "I386SysVFerror",
    "I386SysVFgetc",
    "I386SysVFgets",
    "I386SysVFopen",
    "I386SysVFprintf",
    "I386SysVFputc",
    "I386SysVFputs",
    "I386SysVFread",
    "I386SysVFscanf",
    "I386SysVFseek",
    "I386SysVFtell",
    "I386SysVFwrite",
    "I386SysVGetc",
    "I386SysVGetchar",
    "I386SysVGets",
    "I386SysVPrintf",
    "I386SysVPutc",
    "I386SysVPutchar",
    "I386SysVPuts",
    "I386SysVRemove",
    "I386SysVRename",
    "I386SysVRewind",
    "I386SysVScanf",
    "I386SysVSnprintf",
    "I386SysVSprintf",
    "I386SysVSscanf",
    "I386SysVClearerr",
    "I386SysVFflush",
    "I386SysVFreopen",
    "I386SysVFgetpos",
    "I386SysVFsetpos",
    "I386SysVTmpfile",
    "I386SysVTmpnam",
    "I386SysVUngetc",
    "I386SysVVfprintf",
    "I386SysVVfscanf",
    "I386SysVVprintf",
    "I386SysVVscanf",
    "I386SysVVsnprintf",
    "I386SysVVsprintf",
    "I386SysVVsscanf",
]
