from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import MIPS64SysVModel


class MIPS64SysVFclose(Fclose, MIPS64SysVModel):
    pass


class MIPS64SysVFeof(Feof, MIPS64SysVModel):
    pass


class MIPS64SysVFerror(Ferror, MIPS64SysVModel):
    pass


class MIPS64SysVFgetc(Fgetc, MIPS64SysVModel):
    pass


class MIPS64SysVFgets(Fgets, MIPS64SysVModel):
    pass


class MIPS64SysVFopen(Fopen, MIPS64SysVModel):
    pass


class MIPS64SysVFprintf(Fprintf, MIPS64SysVModel):
    pass


class MIPS64SysVFputc(Fputc, MIPS64SysVModel):
    pass


class MIPS64SysVFputs(Fputs, MIPS64SysVModel):
    pass


class MIPS64SysVFread(Fread, MIPS64SysVModel):
    pass


class MIPS64SysVFscanf(Fscanf, MIPS64SysVModel):
    pass


class MIPS64SysVFseek(Fseek, MIPS64SysVModel):
    pass


class MIPS64SysVFtell(Ftell, MIPS64SysVModel):
    pass


class MIPS64SysVFwrite(Fwrite, MIPS64SysVModel):
    pass


class MIPS64SysVGetc(Getc, MIPS64SysVModel):
    pass


class MIPS64SysVGetchar(Getchar, MIPS64SysVModel):
    pass


class MIPS64SysVGets(Gets, MIPS64SysVModel):
    pass


class MIPS64SysVPrintf(Printf, MIPS64SysVModel):
    pass


class MIPS64SysVPutc(Putc, MIPS64SysVModel):
    pass


class MIPS64SysVPutchar(Putchar, MIPS64SysVModel):
    pass


class MIPS64SysVPuts(Puts, MIPS64SysVModel):
    pass


class MIPS64SysVRemove(Remove, MIPS64SysVModel):
    pass


class MIPS64SysVRename(Rename, MIPS64SysVModel):
    pass


class MIPS64SysVRewind(Rewind, MIPS64SysVModel):
    pass


class MIPS64SysVScanf(Scanf, MIPS64SysVModel):
    pass


class MIPS64SysVSnprintf(Snprintf, MIPS64SysVModel):
    pass


class MIPS64SysVSprintf(Sprintf, MIPS64SysVModel):
    pass


class MIPS64SysVSscanf(Sscanf, MIPS64SysVModel):
    pass


class MIPS64SysVClearerr(Clearerr, MIPS64SysVModel):
    pass


class MIPS64SysVFflush(Fflush, MIPS64SysVModel):
    pass


class MIPS64SysVFreopen(Freopen, MIPS64SysVModel):
    pass


class MIPS64SysVFgetpos(Fgetpos, MIPS64SysVModel):
    pass


class MIPS64SysVFsetpos(Fsetpos, MIPS64SysVModel):
    pass


class MIPS64SysVTmpfile(Tmpfile, MIPS64SysVModel):
    pass


class MIPS64SysVTmpnam(Tmpnam, MIPS64SysVModel):
    pass


class MIPS64SysVUngetc(Ungetc, MIPS64SysVModel):
    pass


class MIPS64SysVVfprintf(Vfprintf, MIPS64SysVModel):
    pass


class MIPS64SysVVfscanf(Vfscanf, MIPS64SysVModel):
    pass


class MIPS64SysVVprintf(Vprintf, MIPS64SysVModel):
    pass


class MIPS64SysVVscanf(Vscanf, MIPS64SysVModel):
    pass


class MIPS64SysVVsnprintf(Vsnprintf, MIPS64SysVModel):
    pass


class MIPS64SysVVsprintf(Vsprintf, MIPS64SysVModel):
    pass


class MIPS64SysVVsscanf(Vsscanf, MIPS64SysVModel):
    pass


__all__ = [
    "MIPS64SysVFclose",
    "MIPS64SysVFeof",
    "MIPS64SysVFerror",
    "MIPS64SysVFgetc",
    "MIPS64SysVFgets",
    "MIPS64SysVFopen",
    "MIPS64SysVFprintf",
    "MIPS64SysVFputc",
    "MIPS64SysVFputs",
    "MIPS64SysVFread",
    "MIPS64SysVFscanf",
    "MIPS64SysVFseek",
    "MIPS64SysVFtell",
    "MIPS64SysVFwrite",
    "MIPS64SysVGetc",
    "MIPS64SysVGetchar",
    "MIPS64SysVGets",
    "MIPS64SysVPrintf",
    "MIPS64SysVPutc",
    "MIPS64SysVPutchar",
    "MIPS64SysVPuts",
    "MIPS64SysVRemove",
    "MIPS64SysVRename",
    "MIPS64SysVRewind",
    "MIPS64SysVScanf",
    "MIPS64SysVSnprintf",
    "MIPS64SysVSprintf",
    "MIPS64SysVSscanf",
    "MIPS64SysVClearerr",
    "MIPS64SysVFflush",
    "MIPS64SysVFreopen",
    "MIPS64SysVFgetpos",
    "MIPS64SysVFsetpos",
    "MIPS64SysVTmpfile",
    "MIPS64SysVTmpnam",
    "MIPS64SysVUngetc",
    "MIPS64SysVVfprintf",
    "MIPS64SysVVfscanf",
    "MIPS64SysVVprintf",
    "MIPS64SysVVscanf",
    "MIPS64SysVVsnprintf",
    "MIPS64SysVVsprintf",
    "MIPS64SysVVsscanf",
]
