from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import ArmHFSysVModel


class ArmHFSysVFclose(Fclose, ArmHFSysVModel):
    pass


class ArmHFSysVFeof(Feof, ArmHFSysVModel):
    pass


class ArmHFSysVFerror(Ferror, ArmHFSysVModel):
    pass


class ArmHFSysVFgetc(Fgetc, ArmHFSysVModel):
    pass


class ArmHFSysVFgets(Fgets, ArmHFSysVModel):
    pass


class ArmHFSysVFopen(Fopen, ArmHFSysVModel):
    pass


class ArmHFSysVFprintf(Fprintf, ArmHFSysVModel):
    pass


class ArmHFSysVFputc(Fputc, ArmHFSysVModel):
    pass


class ArmHFSysVFputs(Fputs, ArmHFSysVModel):
    pass


class ArmHFSysVFread(Fread, ArmHFSysVModel):
    pass


class ArmHFSysVFscanf(Fscanf, ArmHFSysVModel):
    pass


class ArmHFSysVFseek(Fseek, ArmHFSysVModel):
    pass


class ArmHFSysVFtell(Ftell, ArmHFSysVModel):
    pass


class ArmHFSysVFwrite(Fwrite, ArmHFSysVModel):
    pass


class ArmHFSysVGetc(Getc, ArmHFSysVModel):
    pass


class ArmHFSysVGetchar(Getchar, ArmHFSysVModel):
    pass


class ArmHFSysVGets(Gets, ArmHFSysVModel):
    pass


class ArmHFSysVPrintf(Printf, ArmHFSysVModel):
    pass


class ArmHFSysVPutc(Putc, ArmHFSysVModel):
    pass


class ArmHFSysVPutchar(Putchar, ArmHFSysVModel):
    pass


class ArmHFSysVPuts(Puts, ArmHFSysVModel):
    pass


class ArmHFSysVRemove(Remove, ArmHFSysVModel):
    pass


class ArmHFSysVRename(Rename, ArmHFSysVModel):
    pass


class ArmHFSysVRewind(Rewind, ArmHFSysVModel):
    pass


class ArmHFSysVScanf(Scanf, ArmHFSysVModel):
    pass


class ArmHFSysVSnprintf(Snprintf, ArmHFSysVModel):
    pass


class ArmHFSysVSprintf(Sprintf, ArmHFSysVModel):
    pass


class ArmHFSysVSscanf(Sscanf, ArmHFSysVModel):
    pass


class ArmHFSysVClearerr(Clearerr, ArmHFSysVModel):
    pass


class ArmHFSysVFflush(Fflush, ArmHFSysVModel):
    pass


class ArmHFSysVFreopen(Freopen, ArmHFSysVModel):
    pass


class ArmHFSysVFgetpos(Fgetpos, ArmHFSysVModel):
    pass


class ArmHFSysVFsetpos(Fsetpos, ArmHFSysVModel):
    pass


class ArmHFSysVTmpfile(Tmpfile, ArmHFSysVModel):
    pass


class ArmHFSysVTmpnam(Tmpnam, ArmHFSysVModel):
    pass


class ArmHFSysVUngetc(Ungetc, ArmHFSysVModel):
    pass


class ArmHFSysVVfprintf(Vfprintf, ArmHFSysVModel):
    pass


class ArmHFSysVVfscanf(Vfscanf, ArmHFSysVModel):
    pass


class ArmHFSysVVprintf(Vprintf, ArmHFSysVModel):
    pass


class ArmHFSysVVscanf(Vscanf, ArmHFSysVModel):
    pass


class ArmHFSysVVsnprintf(Vsnprintf, ArmHFSysVModel):
    pass


class ArmHFSysVVsprintf(Vsprintf, ArmHFSysVModel):
    pass


class ArmHFSysVVsscanf(Vsscanf, ArmHFSysVModel):
    pass


__all__ = [
    "ArmHFSysVFclose",
    "ArmHFSysVFeof",
    "ArmHFSysVFerror",
    "ArmHFSysVFgetc",
    "ArmHFSysVFgets",
    "ArmHFSysVFopen",
    "ArmHFSysVFprintf",
    "ArmHFSysVFputc",
    "ArmHFSysVFputs",
    "ArmHFSysVFread",
    "ArmHFSysVFscanf",
    "ArmHFSysVFseek",
    "ArmHFSysVFtell",
    "ArmHFSysVFwrite",
    "ArmHFSysVGetc",
    "ArmHFSysVGetchar",
    "ArmHFSysVGets",
    "ArmHFSysVPrintf",
    "ArmHFSysVPutc",
    "ArmHFSysVPutchar",
    "ArmHFSysVPuts",
    "ArmHFSysVRemove",
    "ArmHFSysVRename",
    "ArmHFSysVRewind",
    "ArmHFSysVScanf",
    "ArmHFSysVSnprintf",
    "ArmHFSysVSprintf",
    "ArmHFSysVSscanf",
    "ArmHFSysVClearerr",
    "ArmHFSysVFflush",
    "ArmHFSysVFreopen",
    "ArmHFSysVFgetpos",
    "ArmHFSysVFsetpos",
    "ArmHFSysVTmpfile",
    "ArmHFSysVTmpnam",
    "ArmHFSysVUngetc",
    "ArmHFSysVVfprintf",
    "ArmHFSysVVfscanf",
    "ArmHFSysVVprintf",
    "ArmHFSysVVscanf",
    "ArmHFSysVVsnprintf",
    "ArmHFSysVVsprintf",
    "ArmHFSysVVsscanf",
]
