from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import AArch64SysVModel


class AArch64SysVFclose(Fclose, AArch64SysVModel):
    pass


class AArch64SysVFeof(Feof, AArch64SysVModel):
    pass


class AArch64SysVFerror(Ferror, AArch64SysVModel):
    pass


class AArch64SysVFgetc(Fgetc, AArch64SysVModel):
    pass


class AArch64SysVFgets(Fgets, AArch64SysVModel):
    pass


class AArch64SysVFopen(Fopen, AArch64SysVModel):
    pass


class AArch64SysVFprintf(Fprintf, AArch64SysVModel):
    pass


class AArch64SysVFputc(Fputc, AArch64SysVModel):
    pass


class AArch64SysVFputs(Fputs, AArch64SysVModel):
    pass


class AArch64SysVFread(Fread, AArch64SysVModel):
    pass


class AArch64SysVFscanf(Fscanf, AArch64SysVModel):
    pass


class AArch64SysVFseek(Fseek, AArch64SysVModel):
    pass


class AArch64SysVFtell(Ftell, AArch64SysVModel):
    pass


class AArch64SysVFwrite(Fwrite, AArch64SysVModel):
    pass


class AArch64SysVGetc(Getc, AArch64SysVModel):
    pass


class AArch64SysVGetchar(Getchar, AArch64SysVModel):
    pass


class AArch64SysVGets(Gets, AArch64SysVModel):
    pass


class AArch64SysVPrintf(Printf, AArch64SysVModel):
    pass


class AArch64SysVPutc(Putc, AArch64SysVModel):
    pass


class AArch64SysVPutchar(Putchar, AArch64SysVModel):
    pass


class AArch64SysVPuts(Puts, AArch64SysVModel):
    pass


class AArch64SysVRemove(Remove, AArch64SysVModel):
    pass


class AArch64SysVRename(Rename, AArch64SysVModel):
    pass


class AArch64SysVRewind(Rewind, AArch64SysVModel):
    pass


class AArch64SysVScanf(Scanf, AArch64SysVModel):
    pass


class AArch64SysVSnprintf(Snprintf, AArch64SysVModel):
    pass


class AArch64SysVSprintf(Sprintf, AArch64SysVModel):
    pass


class AArch64SysVSscanf(Sscanf, AArch64SysVModel):
    pass


class AArch64SysVClearerr(Clearerr, AArch64SysVModel):
    pass


class AArch64SysVFflush(Fflush, AArch64SysVModel):
    pass


class AArch64SysVFreopen(Freopen, AArch64SysVModel):
    pass


class AArch64SysVFgetpos(Fgetpos, AArch64SysVModel):
    pass


class AArch64SysVFsetpos(Fsetpos, AArch64SysVModel):
    pass


class AArch64SysVTmpfile(Tmpfile, AArch64SysVModel):
    pass


class AArch64SysVTmpnam(Tmpnam, AArch64SysVModel):
    pass


class AArch64SysVUngetc(Ungetc, AArch64SysVModel):
    pass


class AArch64SysVVfprintf(Vfprintf, AArch64SysVModel):
    pass


class AArch64SysVVfscanf(Vfscanf, AArch64SysVModel):
    pass


class AArch64SysVVprintf(Vprintf, AArch64SysVModel):
    pass


class AArch64SysVVscanf(Vscanf, AArch64SysVModel):
    pass


class AArch64SysVVsnprintf(Vsnprintf, AArch64SysVModel):
    pass


class AArch64SysVVsprintf(Vsprintf, AArch64SysVModel):
    pass


class AArch64SysVVsscanf(Vsscanf, AArch64SysVModel):
    pass


__all__ = [
    "AArch64SysVFclose",
    "AArch64SysVFeof",
    "AArch64SysVFerror",
    "AArch64SysVFgetc",
    "AArch64SysVFgets",
    "AArch64SysVFopen",
    "AArch64SysVFprintf",
    "AArch64SysVFputc",
    "AArch64SysVFputs",
    "AArch64SysVFread",
    "AArch64SysVFscanf",
    "AArch64SysVFseek",
    "AArch64SysVFtell",
    "AArch64SysVFwrite",
    "AArch64SysVGetc",
    "AArch64SysVGetchar",
    "AArch64SysVGets",
    "AArch64SysVPrintf",
    "AArch64SysVPutc",
    "AArch64SysVPutchar",
    "AArch64SysVPuts",
    "AArch64SysVRemove",
    "AArch64SysVRename",
    "AArch64SysVRewind",
    "AArch64SysVScanf",
    "AArch64SysVSnprintf",
    "AArch64SysVSprintf",
    "AArch64SysVSscanf",
    "AArch64SysVClearerr",
    "AArch64SysVFflush",
    "AArch64SysVFreopen",
    "AArch64SysVFgetpos",
    "AArch64SysVFsetpos",
    "AArch64SysVTmpfile",
    "AArch64SysVTmpnam",
    "AArch64SysVUngetc",
    "AArch64SysVVfprintf",
    "AArch64SysVVfscanf",
    "AArch64SysVVprintf",
    "AArch64SysVVscanf",
    "AArch64SysVVsnprintf",
    "AArch64SysVVsprintf",
    "AArch64SysVVsscanf",
]
