from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import ArmELSysVModel


class ArmELSysVFclose(Fclose, ArmELSysVModel):
    pass


class ArmELSysVFeof(Feof, ArmELSysVModel):
    pass


class ArmELSysVFerror(Ferror, ArmELSysVModel):
    pass


class ArmELSysVFgetc(Fgetc, ArmELSysVModel):
    pass


class ArmELSysVFgets(Fgets, ArmELSysVModel):
    pass


class ArmELSysVFopen(Fopen, ArmELSysVModel):
    pass


class ArmELSysVFprintf(Fprintf, ArmELSysVModel):
    pass


class ArmELSysVFputc(Fputc, ArmELSysVModel):
    pass


class ArmELSysVFputs(Fputs, ArmELSysVModel):
    pass


class ArmELSysVFread(Fread, ArmELSysVModel):
    pass


class ArmELSysVFscanf(Fscanf, ArmELSysVModel):
    pass


class ArmELSysVFseek(Fseek, ArmELSysVModel):
    pass


class ArmELSysVFtell(Ftell, ArmELSysVModel):
    pass


class ArmELSysVFwrite(Fwrite, ArmELSysVModel):
    pass


class ArmELSysVGetc(Getc, ArmELSysVModel):
    pass


class ArmELSysVGetchar(Getchar, ArmELSysVModel):
    pass


class ArmELSysVGets(Gets, ArmELSysVModel):
    pass


class ArmELSysVPrintf(Printf, ArmELSysVModel):
    pass


class ArmELSysVPutc(Putc, ArmELSysVModel):
    pass


class ArmELSysVPutchar(Putchar, ArmELSysVModel):
    pass


class ArmELSysVPuts(Puts, ArmELSysVModel):
    pass


class ArmELSysVRemove(Remove, ArmELSysVModel):
    pass


class ArmELSysVRename(Rename, ArmELSysVModel):
    pass


class ArmELSysVRewind(Rewind, ArmELSysVModel):
    pass


class ArmELSysVScanf(Scanf, ArmELSysVModel):
    pass


class ArmELSysVSnprintf(Snprintf, ArmELSysVModel):
    pass


class ArmELSysVSprintf(Sprintf, ArmELSysVModel):
    pass


class ArmELSysVSscanf(Sscanf, ArmELSysVModel):
    pass


class ArmELSysVClearerr(Clearerr, ArmELSysVModel):
    pass


class ArmELSysVFflush(Fflush, ArmELSysVModel):
    pass


class ArmELSysVFreopen(Freopen, ArmELSysVModel):
    pass


class ArmELSysVFgetpos(Fgetpos, ArmELSysVModel):
    pass


class ArmELSysVFsetpos(Fsetpos, ArmELSysVModel):
    pass


class ArmELSysVTmpfile(Tmpfile, ArmELSysVModel):
    pass


class ArmELSysVTmpnam(Tmpnam, ArmELSysVModel):
    pass


class ArmELSysVUngetc(Ungetc, ArmELSysVModel):
    pass


class ArmELSysVVfprintf(Vfprintf, ArmELSysVModel):
    pass


class ArmELSysVVfscanf(Vfscanf, ArmELSysVModel):
    pass


class ArmELSysVVprintf(Vprintf, ArmELSysVModel):
    pass


class ArmELSysVVscanf(Vscanf, ArmELSysVModel):
    pass


class ArmELSysVVsnprintf(Vsnprintf, ArmELSysVModel):
    pass


class ArmELSysVVsprintf(Vsprintf, ArmELSysVModel):
    pass


class ArmELSysVVsscanf(Vsscanf, ArmELSysVModel):
    pass


__all__ = [
    "ArmELSysVFclose",
    "ArmELSysVFeof",
    "ArmELSysVFerror",
    "ArmELSysVFgetc",
    "ArmELSysVFgets",
    "ArmELSysVFopen",
    "ArmELSysVFprintf",
    "ArmELSysVFputc",
    "ArmELSysVFputs",
    "ArmELSysVFread",
    "ArmELSysVFscanf",
    "ArmELSysVFseek",
    "ArmELSysVFtell",
    "ArmELSysVFwrite",
    "ArmELSysVGetc",
    "ArmELSysVGetchar",
    "ArmELSysVGets",
    "ArmELSysVPrintf",
    "ArmELSysVPutc",
    "ArmELSysVPutchar",
    "ArmELSysVPuts",
    "ArmELSysVRemove",
    "ArmELSysVRename",
    "ArmELSysVRewind",
    "ArmELSysVScanf",
    "ArmELSysVSnprintf",
    "ArmELSysVSprintf",
    "ArmELSysVSscanf",
    "ArmELSysVClearerr",
    "ArmELSysVFflush",
    "ArmELSysVFreopen",
    "ArmELSysVFgetpos",
    "ArmELSysVFsetpos",
    "ArmELSysVTmpfile",
    "ArmELSysVTmpnam",
    "ArmELSysVUngetc",
    "ArmELSysVVfprintf",
    "ArmELSysVVfscanf",
    "ArmELSysVVprintf",
    "ArmELSysVVscanf",
    "ArmELSysVVsnprintf",
    "ArmELSysVVsprintf",
    "ArmELSysVVsscanf",
]
