from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import MIPS64ELSysVModel


class MIPS64ELSysVFclose(Fclose, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFeof(Feof, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFerror(Ferror, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFgetc(Fgetc, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFgets(Fgets, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFopen(Fopen, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFprintf(Fprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFputc(Fputc, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFputs(Fputs, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFread(Fread, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFscanf(Fscanf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFseek(Fseek, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFtell(Ftell, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFwrite(Fwrite, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVGetc(Getc, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVGetchar(Getchar, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVGets(Gets, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVPrintf(Printf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVPutc(Putc, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVPutchar(Putchar, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVPuts(Puts, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVRemove(Remove, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVRename(Rename, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVRewind(Rewind, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVScanf(Scanf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVSnprintf(Snprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVSprintf(Sprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVSscanf(Sscanf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVClearerr(Clearerr, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFflush(Fflush, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFreopen(Freopen, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFgetpos(Fgetpos, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVFsetpos(Fsetpos, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVTmpfile(Tmpfile, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVTmpnam(Tmpnam, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVUngetc(Ungetc, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVfprintf(Vfprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVfscanf(Vfscanf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVprintf(Vprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVscanf(Vscanf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVsnprintf(Vsnprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVsprintf(Vsprintf, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVVsscanf(Vsscanf, MIPS64ELSysVModel):
    pass


__all__ = [
    "MIPS64ELSysVFclose",
    "MIPS64ELSysVFeof",
    "MIPS64ELSysVFerror",
    "MIPS64ELSysVFgetc",
    "MIPS64ELSysVFgets",
    "MIPS64ELSysVFopen",
    "MIPS64ELSysVFprintf",
    "MIPS64ELSysVFputc",
    "MIPS64ELSysVFputs",
    "MIPS64ELSysVFread",
    "MIPS64ELSysVFscanf",
    "MIPS64ELSysVFseek",
    "MIPS64ELSysVFtell",
    "MIPS64ELSysVFwrite",
    "MIPS64ELSysVGetc",
    "MIPS64ELSysVGetchar",
    "MIPS64ELSysVGets",
    "MIPS64ELSysVPrintf",
    "MIPS64ELSysVPutc",
    "MIPS64ELSysVPutchar",
    "MIPS64ELSysVPuts",
    "MIPS64ELSysVRemove",
    "MIPS64ELSysVRename",
    "MIPS64ELSysVRewind",
    "MIPS64ELSysVScanf",
    "MIPS64ELSysVSnprintf",
    "MIPS64ELSysVSprintf",
    "MIPS64ELSysVSscanf",
    "MIPS64ELSysVClearerr",
    "MIPS64ELSysVFflush",
    "MIPS64ELSysVFreopen",
    "MIPS64ELSysVFgetpos",
    "MIPS64ELSysVFsetpos",
    "MIPS64ELSysVTmpfile",
    "MIPS64ELSysVTmpnam",
    "MIPS64ELSysVUngetc",
    "MIPS64ELSysVVfprintf",
    "MIPS64ELSysVVfscanf",
    "MIPS64ELSysVVprintf",
    "MIPS64ELSysVVscanf",
    "MIPS64ELSysVVsnprintf",
    "MIPS64ELSysVVsprintf",
    "MIPS64ELSysVVsscanf",
]
