from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import LoongArch64SysVModel


class LoongArch64SysVFclose(Fclose, LoongArch64SysVModel):
    pass


class LoongArch64SysVFeof(Feof, LoongArch64SysVModel):
    pass


class LoongArch64SysVFerror(Ferror, LoongArch64SysVModel):
    pass


class LoongArch64SysVFgetc(Fgetc, LoongArch64SysVModel):
    pass


class LoongArch64SysVFgets(Fgets, LoongArch64SysVModel):
    pass


class LoongArch64SysVFopen(Fopen, LoongArch64SysVModel):
    pass


class LoongArch64SysVFprintf(Fprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVFputc(Fputc, LoongArch64SysVModel):
    pass


class LoongArch64SysVFputs(Fputs, LoongArch64SysVModel):
    pass


class LoongArch64SysVFread(Fread, LoongArch64SysVModel):
    pass


class LoongArch64SysVFscanf(Fscanf, LoongArch64SysVModel):
    pass


class LoongArch64SysVFseek(Fseek, LoongArch64SysVModel):
    pass


class LoongArch64SysVFtell(Ftell, LoongArch64SysVModel):
    pass


class LoongArch64SysVFwrite(Fwrite, LoongArch64SysVModel):
    pass


class LoongArch64SysVGetc(Getc, LoongArch64SysVModel):
    pass


class LoongArch64SysVGetchar(Getchar, LoongArch64SysVModel):
    pass


class LoongArch64SysVGets(Gets, LoongArch64SysVModel):
    pass


class LoongArch64SysVPrintf(Printf, LoongArch64SysVModel):
    pass


class LoongArch64SysVPutc(Putc, LoongArch64SysVModel):
    pass


class LoongArch64SysVPutchar(Putchar, LoongArch64SysVModel):
    pass


class LoongArch64SysVPuts(Puts, LoongArch64SysVModel):
    pass


class LoongArch64SysVRemove(Remove, LoongArch64SysVModel):
    pass


class LoongArch64SysVRename(Rename, LoongArch64SysVModel):
    pass


class LoongArch64SysVRewind(Rewind, LoongArch64SysVModel):
    pass


class LoongArch64SysVScanf(Scanf, LoongArch64SysVModel):
    pass


class LoongArch64SysVSnprintf(Snprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVSprintf(Sprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVSscanf(Sscanf, LoongArch64SysVModel):
    pass


class LoongArch64SysVClearerr(Clearerr, LoongArch64SysVModel):
    pass


class LoongArch64SysVFflush(Fflush, LoongArch64SysVModel):
    pass


class LoongArch64SysVFreopen(Freopen, LoongArch64SysVModel):
    pass


class LoongArch64SysVFgetpos(Fgetpos, LoongArch64SysVModel):
    pass


class LoongArch64SysVFsetpos(Fsetpos, LoongArch64SysVModel):
    pass


class LoongArch64SysVTmpfile(Tmpfile, LoongArch64SysVModel):
    pass


class LoongArch64SysVTmpnam(Tmpnam, LoongArch64SysVModel):
    pass


class LoongArch64SysVUngetc(Ungetc, LoongArch64SysVModel):
    pass


class LoongArch64SysVVfprintf(Vfprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVVfscanf(Vfscanf, LoongArch64SysVModel):
    pass


class LoongArch64SysVVprintf(Vprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVVscanf(Vscanf, LoongArch64SysVModel):
    pass


class LoongArch64SysVVsnprintf(Vsnprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVVsprintf(Vsprintf, LoongArch64SysVModel):
    pass


class LoongArch64SysVVsscanf(Vsscanf, LoongArch64SysVModel):
    pass


__all__ = [
    "LoongArch64SysVFclose",
    "LoongArch64SysVFeof",
    "LoongArch64SysVFerror",
    "LoongArch64SysVFgetc",
    "LoongArch64SysVFgets",
    "LoongArch64SysVFopen",
    "LoongArch64SysVFprintf",
    "LoongArch64SysVFputc",
    "LoongArch64SysVFputs",
    "LoongArch64SysVFread",
    "LoongArch64SysVFscanf",
    "LoongArch64SysVFseek",
    "LoongArch64SysVFtell",
    "LoongArch64SysVFwrite",
    "LoongArch64SysVGetc",
    "LoongArch64SysVGetchar",
    "LoongArch64SysVGets",
    "LoongArch64SysVPrintf",
    "LoongArch64SysVPutc",
    "LoongArch64SysVPutchar",
    "LoongArch64SysVPuts",
    "LoongArch64SysVRemove",
    "LoongArch64SysVRename",
    "LoongArch64SysVRewind",
    "LoongArch64SysVScanf",
    "LoongArch64SysVSnprintf",
    "LoongArch64SysVSprintf",
    "LoongArch64SysVSscanf",
    "LoongArch64SysVClearerr",
    "LoongArch64SysVFflush",
    "LoongArch64SysVFreopen",
    "LoongArch64SysVFgetpos",
    "LoongArch64SysVFsetpos",
    "LoongArch64SysVTmpfile",
    "LoongArch64SysVTmpnam",
    "LoongArch64SysVUngetc",
    "LoongArch64SysVVfprintf",
    "LoongArch64SysVVfscanf",
    "LoongArch64SysVVprintf",
    "LoongArch64SysVVscanf",
    "LoongArch64SysVVsnprintf",
    "LoongArch64SysVVsprintf",
    "LoongArch64SysVVsscanf",
]
