from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import AMD64SysVModel


class AMD64SysVFclose(Fclose, AMD64SysVModel):
    pass


class AMD64SysVFeof(Feof, AMD64SysVModel):
    pass


class AMD64SysVFerror(Ferror, AMD64SysVModel):
    pass


class AMD64SysVFgetc(Fgetc, AMD64SysVModel):
    pass


class AMD64SysVFgets(Fgets, AMD64SysVModel):
    pass


class AMD64SysVFopen(Fopen, AMD64SysVModel):
    pass


class AMD64SysVFprintf(Fprintf, AMD64SysVModel):
    pass


class AMD64SysVFputc(Fputc, AMD64SysVModel):
    pass


class AMD64SysVFputs(Fputs, AMD64SysVModel):
    pass


class AMD64SysVFread(Fread, AMD64SysVModel):
    pass


class AMD64SysVFscanf(Fscanf, AMD64SysVModel):
    pass


class AMD64SysVFseek(Fseek, AMD64SysVModel):
    pass


class AMD64SysVFtell(Ftell, AMD64SysVModel):
    pass


class AMD64SysVFwrite(Fwrite, AMD64SysVModel):
    pass


class AMD64SysVGetc(Getc, AMD64SysVModel):
    pass


class AMD64SysVGetchar(Getchar, AMD64SysVModel):
    pass


class AMD64SysVGets(Gets, AMD64SysVModel):
    pass


class AMD64SysVPrintf(Printf, AMD64SysVModel):
    pass


class AMD64SysVPutc(Putc, AMD64SysVModel):
    pass


class AMD64SysVPutchar(Putchar, AMD64SysVModel):
    pass


class AMD64SysVPuts(Puts, AMD64SysVModel):
    pass


class AMD64SysVRemove(Remove, AMD64SysVModel):
    pass


class AMD64SysVRename(Rename, AMD64SysVModel):
    pass


class AMD64SysVRewind(Rewind, AMD64SysVModel):
    pass


class AMD64SysVScanf(Scanf, AMD64SysVModel):
    pass


class AMD64SysVSnprintf(Snprintf, AMD64SysVModel):
    pass


class AMD64SysVSprintf(Sprintf, AMD64SysVModel):
    pass


class AMD64SysVSscanf(Sscanf, AMD64SysVModel):
    pass


class AMD64SysVClearerr(Clearerr, AMD64SysVModel):
    pass


class AMD64SysVFflush(Fflush, AMD64SysVModel):
    pass


class AMD64SysVFreopen(Freopen, AMD64SysVModel):
    pass


class AMD64SysVFgetpos(Fgetpos, AMD64SysVModel):
    pass


class AMD64SysVFsetpos(Fsetpos, AMD64SysVModel):
    pass


class AMD64SysVTmpfile(Tmpfile, AMD64SysVModel):
    pass


class AMD64SysVTmpnam(Tmpnam, AMD64SysVModel):
    pass


class AMD64SysVUngetc(Ungetc, AMD64SysVModel):
    pass


class AMD64SysVVfprintf(Vfprintf, AMD64SysVModel):
    pass


class AMD64SysVVfscanf(Vfscanf, AMD64SysVModel):
    pass


class AMD64SysVVprintf(Vprintf, AMD64SysVModel):
    pass


class AMD64SysVVscanf(Vscanf, AMD64SysVModel):
    pass


class AMD64SysVVsnprintf(Vsnprintf, AMD64SysVModel):
    pass


class AMD64SysVVsprintf(Vsprintf, AMD64SysVModel):
    pass


class AMD64SysVVsscanf(Vsscanf, AMD64SysVModel):
    pass


__all__ = [
    "AMD64SysVFclose",
    "AMD64SysVFeof",
    "AMD64SysVFerror",
    "AMD64SysVFgetc",
    "AMD64SysVFgets",
    "AMD64SysVFopen",
    "AMD64SysVFprintf",
    "AMD64SysVFputc",
    "AMD64SysVFputs",
    "AMD64SysVFread",
    "AMD64SysVFscanf",
    "AMD64SysVFseek",
    "AMD64SysVFtell",
    "AMD64SysVFwrite",
    "AMD64SysVGetc",
    "AMD64SysVGetchar",
    "AMD64SysVGets",
    "AMD64SysVPrintf",
    "AMD64SysVPutc",
    "AMD64SysVPutchar",
    "AMD64SysVPuts",
    "AMD64SysVRemove",
    "AMD64SysVRename",
    "AMD64SysVRewind",
    "AMD64SysVScanf",
    "AMD64SysVSnprintf",
    "AMD64SysVSprintf",
    "AMD64SysVSscanf",
    "AMD64SysVClearerr",
    "AMD64SysVFflush",
    "AMD64SysVFreopen",
    "AMD64SysVFgetpos",
    "AMD64SysVFsetpos",
    "AMD64SysVTmpfile",
    "AMD64SysVTmpnam",
    "AMD64SysVUngetc",
    "AMD64SysVVfprintf",
    "AMD64SysVVfscanf",
    "AMD64SysVVprintf",
    "AMD64SysVVscanf",
    "AMD64SysVVsnprintf",
    "AMD64SysVVsprintf",
    "AMD64SysVVsscanf",
]
