from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import PowerPCSysVModel


class PowerPCSysVFclose(Fclose, PowerPCSysVModel):
    pass


class PowerPCSysVFeof(Feof, PowerPCSysVModel):
    pass


class PowerPCSysVFerror(Ferror, PowerPCSysVModel):
    pass


class PowerPCSysVFgetc(Fgetc, PowerPCSysVModel):
    pass


class PowerPCSysVFgets(Fgets, PowerPCSysVModel):
    pass


class PowerPCSysVFopen(Fopen, PowerPCSysVModel):
    pass


class PowerPCSysVFprintf(Fprintf, PowerPCSysVModel):
    pass


class PowerPCSysVFputc(Fputc, PowerPCSysVModel):
    pass


class PowerPCSysVFputs(Fputs, PowerPCSysVModel):
    pass


class PowerPCSysVFread(Fread, PowerPCSysVModel):
    pass


class PowerPCSysVFscanf(Fscanf, PowerPCSysVModel):
    pass


class PowerPCSysVFseek(Fseek, PowerPCSysVModel):
    pass


class PowerPCSysVFtell(Ftell, PowerPCSysVModel):
    pass


class PowerPCSysVFwrite(Fwrite, PowerPCSysVModel):
    pass


class PowerPCSysVGetc(Getc, PowerPCSysVModel):
    pass


class PowerPCSysVGetchar(Getchar, PowerPCSysVModel):
    pass


class PowerPCSysVGets(Gets, PowerPCSysVModel):
    pass


class PowerPCSysVPrintf(Printf, PowerPCSysVModel):
    pass


class PowerPCSysVPutc(Putc, PowerPCSysVModel):
    pass


class PowerPCSysVPutchar(Putchar, PowerPCSysVModel):
    pass


class PowerPCSysVPuts(Puts, PowerPCSysVModel):
    pass


class PowerPCSysVRemove(Remove, PowerPCSysVModel):
    pass


class PowerPCSysVRename(Rename, PowerPCSysVModel):
    pass


class PowerPCSysVRewind(Rewind, PowerPCSysVModel):
    pass


class PowerPCSysVScanf(Scanf, PowerPCSysVModel):
    pass


class PowerPCSysVSnprintf(Snprintf, PowerPCSysVModel):
    pass


class PowerPCSysVSprintf(Sprintf, PowerPCSysVModel):
    pass


class PowerPCSysVSscanf(Sscanf, PowerPCSysVModel):
    pass


class PowerPCSysVClearerr(Clearerr, PowerPCSysVModel):
    pass


class PowerPCSysVFflush(Fflush, PowerPCSysVModel):
    pass


class PowerPCSysVFreopen(Freopen, PowerPCSysVModel):
    pass


class PowerPCSysVFgetpos(Fgetpos, PowerPCSysVModel):
    pass


class PowerPCSysVFsetpos(Fsetpos, PowerPCSysVModel):
    pass


class PowerPCSysVTmpfile(Tmpfile, PowerPCSysVModel):
    pass


class PowerPCSysVTmpnam(Tmpnam, PowerPCSysVModel):
    pass


class PowerPCSysVUngetc(Ungetc, PowerPCSysVModel):
    pass


class PowerPCSysVVfprintf(Vfprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVfscanf(Vfscanf, PowerPCSysVModel):
    pass


class PowerPCSysVVprintf(Vprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVscanf(Vscanf, PowerPCSysVModel):
    pass


class PowerPCSysVVsnprintf(Vsnprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVsprintf(Vsprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVsscanf(Vsscanf, PowerPCSysVModel):
    pass


__all__ = [
    "PowerPCSysVFclose",
    "PowerPCSysVFeof",
    "PowerPCSysVFerror",
    "PowerPCSysVFgetc",
    "PowerPCSysVFgets",
    "PowerPCSysVFopen",
    "PowerPCSysVFprintf",
    "PowerPCSysVFputc",
    "PowerPCSysVFputs",
    "PowerPCSysVFread",
    "PowerPCSysVFscanf",
    "PowerPCSysVFseek",
    "PowerPCSysVFtell",
    "PowerPCSysVFwrite",
    "PowerPCSysVGetc",
    "PowerPCSysVGetchar",
    "PowerPCSysVGets",
    "PowerPCSysVPrintf",
    "PowerPCSysVPutc",
    "PowerPCSysVPutchar",
    "PowerPCSysVPuts",
    "PowerPCSysVRemove",
    "PowerPCSysVRename",
    "PowerPCSysVRewind",
    "PowerPCSysVScanf",
    "PowerPCSysVSnprintf",
    "PowerPCSysVSprintf",
    "PowerPCSysVSscanf",
    "PowerPCSysVClearerr",
    "PowerPCSysVFflush",
    "PowerPCSysVFreopen",
    "PowerPCSysVFgetpos",
    "PowerPCSysVFsetpos",
    "PowerPCSysVTmpfile",
    "PowerPCSysVTmpnam",
    "PowerPCSysVUngetc",
    "PowerPCSysVVfprintf",
    "PowerPCSysVVfscanf",
    "PowerPCSysVVprintf",
    "PowerPCSysVVscanf",
    "PowerPCSysVVsnprintf",
    "PowerPCSysVVsprintf",
    "PowerPCSysVVsscanf",
]
